/*
    OCLTools is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Dr. Zaius
    ClusterChimps.org
*/

// standard utilities and systems includes
#include <stdio.h>
#include <string>
#include <iostream>
#include <openssl/des.h>


#include <boost/program_options/options_description.hpp>
#include <boost/program_options/variables_map.hpp>
#include <boost/program_options/parsers.hpp>

const char* version = "oclcrypt 1.0\nCopyright (C) 2011 ClusterChimps.org\nThis software \
is free (GPLv3). There is NO warranty; not even\nfor MERCHANTABILITY or FITNESS \
FOR A PARTICULAR PURPOSE.\n";

const char* info = "This program [En|De]crypts an input file creating an [En|De]crypted output\n\
file.  While this encryption is not strong enough to stand up against crackers\n\
it will keep people from poking around your program binaries with 'strings' and\n\
pulling out your kernel source.\n";

char *
Encrypt( char *Key, char *Msg, int size)
{

   static char*    Res;
   int             n=0;
   DES_cblock      Key2;
   DES_key_schedule schedule;

   Res = ( char * ) malloc( size );

   /* Prepare the key for use with DES_cfb64_encrypt */
   memcpy( Key2, Key,8);
   DES_set_odd_parity( &Key2 );
   DES_set_key_checked( &Key2, &schedule );

   /* Encryption occurs here */
   DES_cfb64_encrypt( ( unsigned char * ) Msg, ( unsigned char * ) Res,
                      size, &schedule, &Key2, &n, DES_ENCRYPT );

   return (Res);

}

char *
Decrypt( char *Key, char *Msg, int size)
{

   static char*    Res;
   int             n=0;

   DES_cblock      Key2;
   DES_key_schedule schedule;

   Res = ( char * ) malloc( size );

   /* Prepare the key for use with DES_cfb64_encrypt */
   memcpy( Key2, Key,8);
   DES_set_odd_parity( &Key2 );
   DES_set_key_checked( &Key2, &schedule );

   /* Decryption occurs here */
   DES_cfb64_encrypt( ( unsigned char * ) Msg, ( unsigned char * ) Res,
                           size, &schedule, &Key2, &n, DES_DECRYPT );

   return (Res);

}

////////////////////////////////////////////////////////////////////////////////
// Main program
////////////////////////////////////////////////////////////////////////////////

namespace po = boost::program_options;

int main(int argc, char **argv){

    bool crypt = 0;
    char* key;

    // Declare the supported options.
    po::options_description desc("Usage: crypt [options] input-file\nOptions");
    desc.add_options()
        ("key,k", po::value<std::string >( ),        "Key for [en/de]cryption")
        ("crypt,c",                                  "Encryption flag")
        ("decrypt,d",                                "Decryption flag")
        ("output-file,o",po::value<std::string >( ), "file for crypt output")
        ("input-file",po::value<std::string >( ),    "input file to crypt")
        ("version",                                  "Print out version info and exit.")
        ("help", "produce help message");

    po::positional_options_description p;
    p.add("input-file", -1);

    po::variables_map vm;
    try {
    po::store(po::command_line_parser(argc, argv).
          options(desc).positional(p).run(), vm);

    po::notify(vm);
    } catch (po::error & e) {
       std::cerr << std::endl << e.what() << std::endl << std::endl;
       std::cout << desc << "\n";
       exit(1);
    }

    if (vm.count("version")) {
        std::cout << version << "\n";
        return 1;
    }
    if (vm.count("help")) {
        std::cout << version << "\n";
        std::cout << info << "\n";
        std::cout << desc << "\n";
        return 1;
    }

    if (vm.count("crypt") && vm.count("decrypt"))
    {
       std::cout << "Error: Can't crypt and decrypt at the same time" << std::endl;
       std::cout << desc << "\n";
     exit(1);
    }
    if (!vm.count("crypt") && !vm.count("decrypt"))
    {
       std::cout << "Error: Must supply crypt or decrypt flag" << std::endl;
       std::cout << desc << "\n";
       exit(1);
    }

    if (!vm.count("key"))
    {
       std::cout << "Error: Must supply a key" << std::endl;
       std::cout << desc << "\n";
       exit(1);
    }
    key = (char*)vm["key"].as<std::string>().c_str();

    if (vm.count("crypt"))
       crypt = 1;

    if (!vm.count("input-file"))
    {
       std::cerr << "\nError: must specify input file" << std::endl;
       std::cout << desc << "\n";
       exit(1);
    }
    std::string inFile = vm["input-file"].as<std::string>();

    std::string outFile = "a.out";
    if (vm.count("output-file"))
       outFile = vm["output-file"].as<std::string>();

    FILE* ofp = fopen(outFile.c_str(), "wb");
    FILE* ifp = fopen(inFile.c_str(), "rb");


    char *output;

     if(ifp == 0)
    {
       std::cout << "Input file " << inFile << " not found!" << std::endl;
       exit(1);
    }

    // get the length of the source code
    fseek(ifp, 0, SEEK_END);
    size_t length = ftell(ifp);
    fseek(ifp, 0, SEEK_SET);

    // allocate a buffer for the source code string and read it in
    char* inputText = (char *)malloc(length + 1);
    if (fread(inputText, length, 1, ifp) != 1)
    {
        fclose(ifp);
        free(inputText);
        return 0;
    }

    inputText[length] = '\0';

    output=(char*)malloc(length + 1);

    if(crypt)
       memcpy(output, Encrypt(key, inputText, length), length);
    else
       memcpy(output, Decrypt(key, inputText, length), length);

    fwrite(output, 1, length, ofp);

    fclose(ofp);
    fclose(ifp);
}


